"process.classes" <-
structure(function(n,classes) {

  ##############################################################################
  #                                                                            #
  # Check number of classes by investigating the 'classes' variable.           #
  #                                                                            #
  ##############################################################################

  if (!is.null(classes)) {

    ############################################################################
    #                                                                          #
    # If 'classes' is a 'factor' rearrange the sample rows of 'X' if the       #
    # 'classes' are unordered.                                                 #
    #                                                                          #
    ############################################################################

    if (is.factor(classes)) {

      if (length(classes)!=n) 
        stop("length of 'classes' does not match the number of samples 'n'")
      s  <- nlevels(classes)
      ns <- sapply(levels(classes),function(level) sum(classes==level))
      if (!is.ordered(classes)) idx <- order(classes)

    ############################################################################
    #                                                                          #
    # If 'classes' is a numeric vector take its elements as class sizes of     #
    # sorted sample classes.                                                   #
    #                                                                          #
    ############################################################################

    } else if (is.numeric(classes)) {
      if (sum(classes)!=n)
        stop("sum of 'classes' does not match size of 'X'")
      s  <- length(classes)
      ns <- classes
      idx <- 1:n

    ############################################################################
    #                                                                          #
    # If 'classes' is of other type than 'factor' or 'numeric' as well as      #
    # different from 'NULL' stop the execution.                                #
    #                                                                          #
    ############################################################################

    } else stop("'classes' have to be a 'factor' or a 'numeric' variable")

  ##############################################################################
  #                                                                            #
  # If 'classes' is 'NULL' assume the data to be one sample data.              #
  #                                                                            #
  ##############################################################################

  } else {
    s  <- 1
    ns <- n
    idx  <- 1:n
  }

  ##############################################################################
  #                                                                            #
  # Order the sample classes by increasing class size.                         #
  #                                                                            #
  ##############################################################################

  idx <- idx[order(rep(rank(ns),ns))]
  ns <- sort(ns)

  ##############################################################################
  #                                                                            #
  # Return the number of sample groups, the vector of sample group sizes and   #
  # the order index of the samples.                                            #
  #                                                                            #
  ##############################################################################

  return(list(s=s,ns=ns,idx=idx))
}
, name = "process.classes")
