"process.grpdata" <-
structure(function(grpdata) {

  ##############################################################################
  #                                                                            #
  # If group data is in table-form transform into list-form.                   #
  #                                                                            #
  ##############################################################################

  if (is.matrix(grpdata) || is.data.frame(grpdata))
    grpdata <- SETGEN.grptab2list(grpdata)

  ##############################################################################
  #                                                                            #
  # Check if 'grpdata' has list style.                                         #
  #                                                                            #
  ##############################################################################

  if (is.list(grpdata)) {

    ############################################################################
    #                                                                          #
    # Get number of groups.                                                    #
    #                                                                          #
    ############################################################################

    gnum <- length(grpdata)
    if (gnum<1) 
      stop("'grpdata'-list has to have at least one element")

    ############################################################################
    #                                                                          #
    # Get group lengths.                                                       #
    #                                                                          #
    ############################################################################

    glen <- unlist(lapply(grpdata,length))
    if (sum(glen==0)>0)
      stop("'grpdata'-list contains empty groups")

    ############################################################################
    #                                                                          #
    # Get group names.                                                         #
    #                                                                          #
    ############################################################################

    gnames <- names(grpdata)
    if (is.null(gnames)) {
      gnames <- 1:gnum
      names(grpdata) <- gnames
      warning("'grpdata'-list contains no group names")
    }

    ############################################################################
    #                                                                          #
    # Make index vector.                                                       #
    #                                                                          #
    ############################################################################

    idx <- unlist(grpdata)

  } else {
    stop("grpdata has to be a 'list' or a two-column 'matrix' or 'data.frame'")
  }

  return(list(grpdata=grpdata,gnum=gnum,gnames=gnames,glen=glen,idx=idx))
}
, name = "process.grpdata")
