#include "candidatesets.h"

int* candidatesets(double *X1, double *X2, int n1, int n2, int p, 
                   double r2min1, double r2min2, bool rposonly1, bool rposonly2,
                   bool pivdom, int *clustlen, int *clenoffset, 
                   int *outclens) {

int i, j, k, clens, clustnum, *clustidx;
double dn1, dn2, *xmean1, *xmean2, *xs21, *xs22, sum, sum2, x, xs2i1, xs2i2,
       *xi1, *xi2, *xj1, *xj2, num1, num2, r21, r22;

dn1=(double)n1;
dn2=(double)n2;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory for variable means and variances. Set pointers to last   */
  /* elements. Reduce 'p' by 1 in order to exclude the seed variable.         */
  /*                                                                          */
  /****************************************************************************/

  xmean1=dvector(1,p);
  xs21=dvector(1,p);
  //x2s1=dvector(1,p);

  X1+=n1*p;
  xmean1+=p;
  xs21+=p;
  //x2s1+=p;

  for (i=p; i>0; i--) {
    for (j=n1, sum=0, sum2=0; j>0; j--) {
      x=*X1;
      sum+=x;
      sum2+=x*x;
      X1--;
    }
    *xmean1=sum/dn1;
    *(xs21--)=sum2-dn1*(*xmean1)*(*xmean1);
    //*(x2s1--)=sum2;
    xmean1--;
  }


  xmean2=dvector(1,p);
  xs22=dvector(1,p);
  //x2s2=dvector(1,p);

  X2+=n2*p;
  xmean2+=p;
  xs22+=p;
  //x2s2+=p;

  for (i=p; i>0; i--) {
    for (j=n2, sum=0, sum2=0; j>0; j--) {
      x=*X2;
      sum+=x;
      sum2+=x*x;
      X2--;
    }
    *xmean2=sum/dn2;
    *(xs22--)=sum2-dn2*(*xmean2)*(*xmean2);
    //*(x2s2--)=sum2;
    xmean2--;
  }

  /**************************************************************************/
  /*                                                                        */
  /* Loop over all variables and calculate correlation between the pivot    */
  /* variable and all other variables. Count correlations above correlation */
  /* threshold and store index of correlated variables in 'clustidx'.       */
  /* 'clustnum' is the number of the groups                                 */
  /* (Note: Without the seed variable there are only 'p'-1 variables),      */
  /* 'clenoffset' is the vector of group offsets in 'clustidx', i.e.,       */
  /* 0,0+clustlen[1],0+clustlen[1]+clustlen[2],..,0+..+clustlen[clustnum-1] */
  /*                                                                        */
  /**************************************************************************/

  fprintf(stdout,"\n Creating correlated groups starts\n");
  fflush(stdout);

  clustnum=p;                  // earlier was clustnum=p-1 but no seed here
  //clustlen=lvector(1,clustnum);
  //clenoffset=lvector(1,clustnum);
  clustidx=lvector(1,clustnum);

  for (i=1, clens=0; i<=clustnum; i++) {
    clenoffset[i]=clens;
    //x2si=x2s[i];
    //xmeani=xmean[i];
    xs2i1=xs21[i];
    xs2i2=xs22[i];

    for (j=1; j<=p; j++) {

      /**********************************************************************/
      /*                                                                    */
      /* Dynamically enlarge the allocated memory for clustidx. The         */
      /* enlargement is for 'p' indices. So there may be a memory overhang*/
      /* that has to be corrected at the end of cluster generation.         */
      /*                                                                    */
      /**********************************************************************/

      if (!(clens%(p))) {
        clustidx++;
        clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*(clens+p)));
        clustidx--;
      }

        // Perhaps it is faster to compute the correlation corr(i,i) p times 
        // than to check p*p times for i==j, but the check is done because
        // then it is guaranteed that the pivot variables are in the created 
        // sets. Otherwise, if r2min was for numerical reasons slightly greater 
        // than 1 the pivot variables could be left out.  
        if (i==j) {
	  clustidx[++clens]=j;
	  continue;
        }

      /**********************************************************************/
      /*                                                                    */
      /* If variance dominance of the pivot variable is requested. The      */
      /* variances of all the other variables are inspected first. If the   */
      /* pivot variable has a variance less than the other variable the     */
      /* current loop is skipped.                                           */
      /*                                                                    */
      /**********************************************************************/

      //if (pivdom) if (x2si<x2s[j]) continue;
      if (pivdom) if (xs2i1<xs21[j]) continue;

      xi1=X1+i*n1;
      xj1=X1+j*n1;

      xi2=X2+i*n2;
      xj2=X2+j*n2;

      /**********************************************************************/
      /*                                                                    */
      /* Sum up xi*yi.                                                      */
      /*                                                                    */
      /**********************************************************************/

      for (k=n1, num1=0.; k>0; k--) num1+=*(xi1--)*(*(xj1--));
      for (k=n2, num2=0.; k>0; k--) num2+=*(xi2--)*(*(xj2--));

      /**********************************************************************/
      /*                                                                    */
      /* Calculate the SQUARED PEARSON correlation coefficient.             */
      /* The coefficient is computed in two stages:                         */
      /*   First the numerator of the unsquared correlation coefficient     */
      /* is computed. The sign of the numerator determines the sign of the  */
      /* correlation coefficient.                                           */
      /* If only positive correlations should be taken into account 'r' is  */
      /* compared with zero. If 'rposonly' equals 'TRUE' but the correlation*/
      /* coefficient is negative the current loop repetition is skipped.    */
      /*   Second, the numerator is multiplied with itself and divided by   */
      /* squared denominator of the correlation coefficient                 */
      /*                                                                    */
      /**********************************************************************/

      //r2=(sum-dn*xmeani*xmean[j]);

      r21=num1;
      if (rposonly1) if (r21<0) continue;
      r21*=r21/(xs2i1*xs21[j]);
      //fprintf(stdout, "\n r21 = %lf ", r21);      

      //r2*=r2/(x2s[i]*x2s[j]);
      
      r22=num2;
      if (rposonly2) if (r22<0) continue;
      r22*=r22/(xs2i2*xs22[j]);
      //fprintf(stdout, " r22 = %lf ", r22);      
      
      if ((r21>=r2min1) && (r22>=r2min2)) clustidx[++clens]=j;
    } 
    clustlen[i]=clens-clenoffset[i];
    //fprintf(stdout, "\n");
  }

  /**************************************************************************/
  /*
  /* return the length of the vector with gene indices
  /*
  /**************************************************************************/

  *outclens = clens;

  /**************************************************************************/
  /*                                                                        */
  /* Fit cluster variable memory allocation.                                */
  /*                                                                        */
  /**************************************************************************/

  fprintf(stdout, "\n Fitting cluster variable memory allocation starts \n");
  fflush(stdout);


  clustidx++;
  clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*clens));
  clustidx--;

  /***************************************************************************/
  /*
  /* free memory
  /*
  /***************************************************************************/
 
  free_dvector(xmean1,1); free_dvector(xmean2,1);
  free_dvector(xs21,1); free_dvector(xs22,1);

 return clustidx;
}
