#include "candidatesets_R.h"

SEXP candidatesets_R(SEXP X1, SEXP X2, SEXP r2min1, SEXP r2min2, SEXP rposonly1, 
                     SEXP rposonly2, SEXP pivdom) {

int *x1dims, *x2dims, *clustlen, *clenoffset, *clustidx, 
     *ptr_rclustidx, *ptr_rclustlen, clens, i,
     n1, n2, p;                       // *ptr_rclenoffset;
double *ptr_X1, *ptr_X2;
SEXP rclustidx, rclustlen, rlist;     //rclenoffset;

// extract dim attributes from the R matrices X1 and X2
x1dims = (int*)INTEGER(coerceVector(getAttrib(X1, R_DimSymbol), INTSXP));
x2dims = (int*)INTEGER(coerceVector(getAttrib(X2, R_DimSymbol), INTSXP));
n1 = x1dims[0]; p = x1dims[1];
n2 = x2dims[0];

// transform R matrices to vectors
PROTECT(X1 = coerceVector(X1, REALSXP));
PROTECT(X2 = coerceVector(X2, REALSXP));
ptr_X1 = REAL(X1);
ptr_X2 = REAL(X2);

clustlen = lvector(1,p);
clenoffset = lvector(1,p);

// candidatesets returns a vector of gene indices in the sets
clustidx = candidatesets(ptr_X1-1, ptr_X2-1, n1, n2, 
               p, NUMERIC_VALUE(r2min1), NUMERIC_VALUE(r2min2),
               INTEGER_VALUE(rposonly1), INTEGER_VALUE(rposonly2), 
               INTEGER_VALUE(pivdom),
               clustlen, clenoffset, &clens);
 
// clustidx actually points one element back because of 1..n arrays. 
// use 0..n arrays here
//clustidx++;

//for(i=1;i<=clens;i++)
//  fprintf(stdout, "\n %ld  \n", clustidx[i]);

//fprintf(stdout, "\n\n");

//for(i=1;i<=p;i++)
//  fprintf(stdout, "\n %ld  \n", clustlen[i]);

//fprintf(stdout, "\n\n");

//for(i=1;i<=p;i++)
//  fprintf(stdout, "\n %ld  \n", clenoffset[i]);


// Allocate memory for the R vectors
PROTECT(rclustidx = NEW_INTEGER(clens));
PROTECT(rclustlen = NEW_INTEGER(p));
//PROTECT(rclenoffset = NEW_INTEGER(p));

// extract pointers to the R vectors to access them from C
ptr_rclustidx = (int*)INTEGER_POINTER(rclustidx);
ptr_rclustlen = (int*)INTEGER_POINTER(rclustlen);
//ptr_rclenoffset = (int*)INTEGER_POINTER(rclenoffset);

/******************************************************************************/
/*
/* copy the returned C vector to R vector. One could try to program more 
/* efficiently and omit this step but it is necessary if one wants to have 
/* a pure C function and a wrapper with the interface to R as here
/*
/******************************************************************************/

for (i=0; i<clens; i++)
  ptr_rclustidx[i] = clustidx[i+1];

for (i=0; i<p; i++)
  ptr_rclustlen[i] = clustlen[i+1];

//for (i=0; i<p; i++)
//  ptr_rclenoffset[i] = clenoffset[i+1];


// free memory from the C vectors;
// for clustidx go back to 1..n arrays, to use the function free_lvector
free_lvector(clustidx,1);
free_lvector(clustlen,1);
free_lvector(clenoffset,1);

// creating a list with 3 vector elements:
PROTECT(rlist = allocVector(VECSXP, 2));
// attaching rclustidx vector to rlist and so on:
SET_VECTOR_ELT(rlist, 0, rclustidx);
SET_VECTOR_ELT(rlist, 1, rclustlen);
//SET_VECTOR_ELT(rlist, 2, rclenoffset);
UNPROTECT(5);

return rlist;
}

