#include "mathutils.h"
#include "computebetasum.h"

/******************************************************************************/
/*                                                                            */
/* Functions to compute the sum of beta statistics                            */
/*                                                                            */
/******************************************************************************/

double compute_betasum(double *X, int n, int p, double *xp, int *clustidx,
    int clustlen, int clenoffset, double *x2s, int pivotidx,
    int direction, bool discret, bool trunc, double ucrit, double minbeta,
    bool samedirect) {

    int j, k, clustidxj;
    double sum, sum2, *xj, r2, sumpiv, *xpiv;
    sum2 = 0.;

    xpiv=X+n*pivotidx;
    xp=xp+n;
    for (k=n, sumpiv=0.; k>0; k--) sumpiv+=*(xp--)*(*(xpiv--));

    if (direction != 0) 
        if ((direction==1 && sumpiv<0) || (direction ==-1 && sumpiv>0))
            return 0.;

    for (j=clustlen, sum2=0.; j>0; j--) {
      xp=xp+n;
      clustidxj=clustidx[clenoffset+j];
      xj=X+n*clustidxj;
      for (k=n, sum=0.; k>0; k--) sum+=*(xp--)*(*(xj--));
      if (samedirect) 
          if (sum*sumpiv<0) continue; 
      r2 = sum*sum/(x2s[p]*x2s[clustidxj]);

      //if (direction != 0) if (clustidxj==pivotidx) {
      //    if ((direction==1 && sum<0) || (direction==-1 && sum>0)) r2=-p;
      //}

      if (discret) {
          r2 = ((r2 < ucrit) || (r2 < minbeta)) ? 0.0 : 1.0;
      } else
      if (trunc) {
          if (r2 < ucrit) r2 = 0.0;
          if (r2 < minbeta) r2 = 0.0;
      }

      sum2+=r2;
    }
 return sum2;
}

/*****************************************************************************/

double compute_betasum_from_maxbeta(double *X, int n, int p, 
    double *xp, int *clustidx, int clustlen, int clenoffset, 
    double *x2s, int pivotidx,
    int direction, bool discret, bool trunc, double ucrit, double minbeta,
    bool samedirect, int maxgene) {

    int j, k, w, clustidxj;
    double sum, sum2, *xj, r2, sumpiv, *xpiv, temp;
    double *tempbetas = dvector(1,clustlen);
    w = 0;
    sum2 = 0.;

    xpiv=X+n*pivotidx;
    xp=xp+n;
    for (k=n, sumpiv=0.; k>0; k--) sumpiv+=*(xp--)*(*(xpiv--));

    if (direction != 0) 
        if ((direction==1 && sumpiv<0) || (direction ==-1 && sumpiv>0))
            return 0.;

    for (j=clustlen; j>0; j--) {
      xp=xp+n;
      clustidxj=clustidx[clenoffset+j];
      xj=X+n*clustidxj;
      for (k=n, sum=0.; k>0; k--) sum+=*(xp--)*(*(xj--));
      if ((samedirect) && (sum*sumpiv<0)) {
          r2 = 0.0;
      } else { 
          r2 = sum*sum/(x2s[p]*x2s[clustidxj]);
      }      

      if (clustidxj==pivotidx) {
          //if (direction != 0)
          //if ((direction==1 && sum<0) || (direction==-1 && sum>0)) r2=-p;
          w = j;
      }
      
      tempbetas[j] = r2;
    }
    if (w == 0) matherror("no pivot variable found");

    temp = tempbetas[w]; 
    tempbetas[w] = tempbetas[clustlen]; 
    tempbetas[clustlen] = temp;
    
    if (maxgene != 1) {
        if (maxgene < clustlen) {
            temp = select_kth_smallest(clustlen+1-maxgene, clustlen-1, tempbetas);
        } else {
            maxgene = clustlen;
        }
    }

    for (j=clustlen, sum2=0.; j>clustlen-maxgene; j--) {
      r2 = tempbetas[j];

      if (discret) {
          r2 = ((r2 < ucrit) || (r2 < minbeta)) ? 0.0 : 1.0;
      } else
      if (trunc) {
          if (r2 < ucrit) r2 = 0.0;
          if (r2 < minbeta) r2 = 0.0;
      }

      sum2+=r2;
    }

    free_dvector(tempbetas,1);

 return sum2;
}
